@@ -1,5 +1,5 @@
 /* gsasl.h --- Header file for GNU SASL Library.
- * Copyright (C) 2002-2020 Simon Josefsson
+ * Copyright (C) 2002-2021 Simon Josefsson
  *
  * This file is part of GNU SASL Library.
  *
@@ -23,10 +23,29 @@
 #ifndef GSASL_H
 #define GSASL_H
 
+/**
+ * SECTION:gsasl
+ * @title: gsasl.h
+ * @short_description: main library interfaces
+ *
+ * The main library interfaces are declared in gsasl.h.
+ */
+
 #include <stdio.h>		/* FILE */
 #include <stddef.h>		/* size_t */
 #include <unistd.h>		/* ssize_t */
 
+/* Get version symbols. */
+#include <gsasl-version.h>
+
+/**
+ * GSASL_API:
+ *
+ * Symbol holding shared library API visibility decorator.
+ *
+ * This is used internally by the library header file and should never
+ * be used or modified by the application.
+ */
 #ifndef GSASL_API
 #if defined GSASL_BUILDING && defined HAVE_VISIBILITY && HAVE_VISIBILITY
 #define GSASL_API __attribute__((__visibility__("default")))
@@ -44,59 +63,6 @@ extern "C"
 {
 #endif
 
-  /**
-   * GSASL_VERSION
-   *
-   * Pre-processor symbol with a string that describe the header file
-   * version number.  Used together with gsasl_check_version() to
-   * verify header file and run-time library consistency.
-   */
-#define GSASL_VERSION "1.9.1"
-
-  /**
-   * GSASL_VERSION_MAJOR
-   *
-   * Pre-processor symbol with a decimal value that describe the major
-   * level of the header file version number.  For example, when the
-   * header version is 1.2.3 this symbol will be 1.
-   *
-   * Since: 1.1
-   */
-#define GSASL_VERSION_MAJOR 1
-
-  /**
-   * GSASL_VERSION_MINOR
-   *
-   * Pre-processor symbol with a decimal value that describe the minor
-   * level of the header file version number.  For example, when the
-   * header version is 1.2.3 this symbol will be 2.
-   *
-   * Since: 1.1
-   */
-#define GSASL_VERSION_MINOR 9
-
-  /**
-   * GSASL_VERSION_PATCH
-   *
-   * Pre-processor symbol with a decimal value that describe the patch
-   * level of the header file version number.  For example, when the
-   * header version is 1.2.3 this symbol will be 3.
-   *
-   * Since: 1.1
-   */
-#define GSASL_VERSION_PATCH 1
-
-  /**
-   * GSASL_VERSION_NUMBER
-   *
-   * Pre-processor symbol with a hexadecimal value describing the
-   * header file version number.  For example, when the header version
-   * is 1.2.3 this symbol will have the value 0x010203.
-   *
-   * Since: 1.1
-   */
-#define GSASL_VERSION_NUMBER 0x010901
-
   /* RFC 2222: SASL mechanisms are named by strings, from 1 to 20
    * characters in length, consisting of upper-case letters, digits,
    * hyphens, and/or underscores.  SASL mechanism names must be
@@ -156,9 +122,6 @@ extern "C"
    * @GSASL_GSSAPI_INQUIRE_MECH_FOR_SASLNAME_ERROR: GSS-API library call error.
    * @GSASL_GSSAPI_TEST_OID_SET_MEMBER_ERROR: GSS-API library call error.
    * @GSASL_GSSAPI_RELEASE_OID_SET_ERROR: GSS-API library call error.
-   * @GSASL_KERBEROS_V5_INIT_ERROR: Init error in KERBEROS_V5.
-   * @GSASL_KERBEROS_V5_INTERNAL_ERROR: General error in KERBEROS_V5.
-   * @GSASL_SHISHI_ERROR: Same as %GSASL_KERBEROS_V5_INTERNAL_ERROR.
    * @GSASL_SECURID_SERVER_NEED_ADDITIONAL_PASSCODE: SecurID mechanism
    *   needs an additional passcode.
    * @GSASL_SECURID_SERVER_NEED_NEW_PIN: SecurID mechanism
@@ -204,9 +167,6 @@ extern "C"
     GSASL_GSSAPI_ACQUIRE_CRED_ERROR = 43,
     GSASL_GSSAPI_DISPLAY_NAME_ERROR = 44,
     GSASL_GSSAPI_UNSUPPORTED_PROTECTION_ERROR = 45,
-    GSASL_KERBEROS_V5_INIT_ERROR = 46,
-    GSASL_KERBEROS_V5_INTERNAL_ERROR = 47,
-    GSASL_SHISHI_ERROR = GSASL_KERBEROS_V5_INTERNAL_ERROR,
     GSASL_SECURID_SERVER_NEED_ADDITIONAL_PASSCODE = 48,
     GSASL_SECURID_SERVER_NEED_NEW_PIN = 49,
     GSASL_GSSAPI_ENCAPSULATE_TOKEN_ERROR = 60,
@@ -415,12 +375,14 @@ extern "C"
   extern GSASL_API void *gsasl_session_hook_get (Gsasl_session * sctx);
 
   /* Property handling: property.c */
-  extern GSASL_API void gsasl_property_set (Gsasl_session * sctx,
-					    Gsasl_property prop,
-					    const char *data);
-  extern GSASL_API void gsasl_property_set_raw (Gsasl_session * sctx,
-						Gsasl_property prop,
-						const char *data, size_t len);
+  extern GSASL_API int gsasl_property_set (Gsasl_session * sctx,
+					   Gsasl_property prop,
+					   const char *data);
+  extern GSASL_API int gsasl_property_set_raw (Gsasl_session * sctx,
+					       Gsasl_property prop,
+					       const char *data, size_t len);
+  extern GSASL_API void gsasl_property_free (Gsasl_session * sctx,
+					     Gsasl_property prop);
   extern GSASL_API const char *gsasl_property_get (Gsasl_session * sctx,
 						   Gsasl_property prop);
   extern GSASL_API const char *gsasl_property_fast (Gsasl_session * sctx,
@@ -467,18 +429,45 @@ extern "C"
 				       int *stringpreprc);
 
   /* Crypto functions: crypto.c */
+
+  /**
+   * Gsasl_hash:
+   * @GSASL_HASH_SHA1: Hash function SHA-1.
+   * @GSASL_HASH_SHA256: Hash function SHA-256.
+   *
+   * Hash functions.  You may use gsasl_hash_length() to get the
+   * output size of a hash function.
+   *
+   * Currently only used as parameter to
+   * gsasl_scram_secrets_from_salted_password() and
+   * gsasl_scram_secrets_from_password() to specify for which SCRAM
+   * mechanism to prepare secrets for.
+   *
+   * Since: 1.10
+   */
   typedef enum
   {
     /* Hash algorithm identifiers. */
-    GSASL_HASH_MD5 = 1,
     GSASL_HASH_SHA1 = 2,
     GSASL_HASH_SHA256 = 3,
   } Gsasl_hash;
 
+  /**
+   * Gsasl_hash_length:
+   * @GSASL_HASH_SHA1_SIZE: Output size of hash function SHA-1.
+   * @GSASL_HASH_SHA256_SIZE: Output size of hash function SHA-256.
+   * @GSASL_HASH_MAX_SIZE: Maximum output size of any %Gsasl_hash_length.
+   *
+   * Identifiers specifying the output size of hash functions.
+   *
+   * These can be used when statically allocating the buffers needed
+   * for, e.g., gsasl_scram_secrets_from_password().
+   *
+   * Since: 1.10
+   */
   typedef enum
   {
     /* Output sizes of hashes. */
-    GSASL_HASH_MD5_SIZE = 16,
     GSASL_HASH_SHA1_SIZE = 20,
     GSASL_HASH_SHA256_SIZE = 32,
     GSASL_HASH_MAX_SIZE = GSASL_HASH_SHA256_SIZE
@@ -522,11 +511,6 @@ extern "C"
   /* Get the mechanism API. */
 #include <gsasl-mech.h>
 
-#ifndef GSASL_NO_OBSOLETE
-  /* For compatibility with earlier versions. */
-#include <gsasl-compat.h>
-#endif
-
 #ifdef __cplusplus
 }
 #endif
